/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2007 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Application
    MRFSimpleFoam

Description
    Steady-state solver for incompressible, turbulent flow of non-Newtonian 
    fluids with MRF regions.
	 Modified to compute the torque on the rotating sufraces

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "incompressible/singlePhaseTransportModel/singlePhaseTransportModel.H"
#include "incompressible/turbulenceModel/turbulenceModel.H"
#include "MRFZonesDrg.H"

void computeTorq(word patchName, fvMesh& mesh, volScalarField& p, const dimensionedVector& Omega, const dimensionedVector& origin);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{

#   include "setRootCase.H"

#   include "createTime.H"
#   include "createMesh.H"
#   include "createFields.H"
#   include "initContinuityErrs.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    Info<< "\nStarting time loop\n" << endl;

    for (runTime++; !runTime.end(); runTime++)
    {
        Info<< "Time = " << runTime.timeName() << nl << endl;

#       include "readSIMPLEControls.H"

        p.storePrevIter();

        // Pressure-velocity SIMPLE corrector
        {
            // Momentum predictor

            tmp<fvVectorMatrix> UEqn
            (
                fvm::div(phi, U)
              + turbulence->divR(U)
            );
            mrfZones.addCoriolis(UEqn());

            UEqn().relax();

            solve(UEqn() == -fvc::grad(p));

            p.boundaryField().updateCoeffs();
            volScalarField rAU = 1.0/UEqn().A();
            U = rAU*UEqn().H();
            UEqn.clear();

            phi = fvc::interpolate(U, "interpolate(HbyA)") & mesh.Sf();
            mrfZones.relativeFlux(phi);
            adjustPhi(phi, U, p);

            // Non-orthogonal pressure corrector loop
            for (int nonOrth=0; nonOrth<=nNonOrthCorr; nonOrth++)
            {
                fvScalarMatrix pEqn
                (
                    fvm::laplacian(rAU, p) == fvc::div(phi)
                );

                pEqn.setReference(pRefCell, pRefValue);
                pEqn.solve();

                if (nonOrth == nNonOrthCorr)
                {
                    phi -= pEqn.flux();
                }
            }

#           include "continuityErrs.H"

            // Explicitly relax pressure for momentum corrector
            p.relax();

            // Momentum corrector
            U -= rAU*fvc::grad(p);
            U.correctBoundaryConditions();
        }

        turbulence->correct();
		  
		  //compute torque
		  forAll(mrfZones,i)
		  {
		  		const wordList& patchNames = mrfZones[i].getPatchNames();
				const dimensionedVector& origin = mrfZones[i].getOrigin();
				const dimensionedVector& Omega = mrfZones[i].getOmega();
				forAll(patchNames,j)
					computeTorq(patchNames[j], mesh, p, Omega, origin);
        }
		  
        runTime.write();

        Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
            << "  ClockTime = " << runTime.elapsedClockTime() << " s"
            << nl << endl;
    }

    Info<< "End\n" << endl;

    return(0);
}

void computeTorq(word patchName, fvMesh& mesh, volScalarField& p, const dimensionedVector& Omega, const dimensionedVector& origin)
{

	// get the label for the requested patch
 	label patchID = mesh.boundaryMesh().findPatchID(patchName);
	
	if (-1 == patchID)
	{
		Info << "no patch named "+patchName << endl <<"Available patches:" << endl;
		error err("Error!\n");

		//access boundary elements
		const fvPatchList& patches = mesh.boundary();
		
		//loop over boundaries
		forAll(patches, patchI)
		{ 	
			Info << patches[patchI].name() << endl;
		}
		
		err.exit();
	}
	
	const fvPatch& cPatch = mesh.boundary()[patchID];
	
//	dimensionedVector torque("torque",dimensionSet(1,2,-2,0,0,0,0),vector::zero);
//	dimensionedScalar pref("pref",dimensionSet(1,-1,-2,0,0,0,0),101325.0);
	vector torque(0,0,0);
	vector r0 = origin.value();
	scalar pref = 101325.0;
	vector omega = Omega.value();
	
/*	forAll(cPatch,faceI)
	{
		scalar pabs = p.boundaryField()[patchID][faceI]+101325;
		vector Area = cPatch.Sf()[faceI];
		vector r = cPatch.Cf()[faceI]-r0;
		vector force = pabs*Area;
		torque = torque+(r^force);
	}
*/	

	torque = gSum((cPatch.Cf()-r0)^((p.boundaryField()[patchID]+pref)*cPatch.Sf()));
	scalar power = torque & omega;
	
	Info << "torque(" << patchName << "): " << torque << "[Nm]; power: " << power << "[W]" << endl;
}

// ************************************************************************* //
